from tests import IntegrationTest

class TestDeleteUserEntry(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.user2_id = self.create_user('user2', self.instru2_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        
    def test_delete_user_entry(self):
        deleted_user_id = self.delete_user(self.user1_id)
        assert deleted_user_id == self.user1_id
        _json = self.get_users()
        assert len(_json['users']) == 1
        assert _json['users'][0]['user']['name'] == 'user2'        



import flask
import sqlite3
from typing import List

from popoll_backend.model import Payload
from popoll_backend.query import Query
from popoll_backend.query.get_user import GetUser


class CreateUser(Query):
    
    name: str
    main_instrument: int
    instruments: List[int]
    
    id: int
    
    def __init__(self, poll: str, name: str, main_instrument: int, instruments: List[int]):
        super().__init__(poll)
        self.name = name
        self.main_instrument = main_instrument
        self.instruments = instruments
    
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('INSERT INTO users(name) VALUES (?)', (self.name,))
        self.id = cursor.lastrowid
        rows = [(self.id, self.main_instrument, True)] + [(self.id, instru, False) for instru in self.instruments if not instru == self.main_instrument]
        cursor.executemany('INSERT INTO user_instruments(user_id, instrument_id, is_main) VALUES(?, ?, ?)', rows)
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return GetUser(self.poll, self.id, details=False).run(cursor)
    
    def error(self, e: sqlite3.Error):
        if isinstance(e, sqlite3.IntegrityError):
            if e.sqlite_errorcode == sqlite3.SQLITE_CONSTRAINT_UNIQUE:
                flask.abort(400, f'User with name {self.name} already exists.')


from tests import IntegrationTest, TestForwardError

class TestDeleteDateEntry(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.user2_id = self.create_user('user2', self.instru2_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.date2_id = self.create_date(date='2025-03-11', time='18:00:00', title='secondDate')
        self.date3_id = self.create_date(date='2025-03-12', time=None, title='thirdDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        self.answer2_id = self.create_answer(user_id=self.user1_id, date_id=self.date2_id)
        self.answer3_id = self.create_answer(user_id=self.user1_id, date_id=self.date3_id)
        self.answer4_id = self.create_answer(user_id=self.user2_id, date_id=self.date1_id)
        self.answer5_id = self.create_answer(user_id=self.user2_id, date_id=self.date2_id)
        self.answer6_id = self.create_answer(user_id=self.user2_id, date_id=self.date3_id)
        
    def test_delete_date_entry(self):
        deleted_date2_id = self.delete_date(self.date2_id)
        deleted_date3_id = self.delete_date(self.date3_id)
        assert deleted_date2_id == self.date2_id
        assert deleted_date3_id == self.date3_id
        _json = self.get_user(self.user1_id)
        assert len(_json['dates']) == 1
        
        
    def test_delete_non_existing_date(self):
        rs = self.delete_date(self.date2_id)
        _json = self.get_user(self.user1_id)
        assert len(_json['dates']) == 2
        rs = self.delete_date(self.date2_id)
        _json = self.get_user(self.user1_id)
        assert len(_json['dates']) == 2


import sqlite3

from popoll_backend.model import Payload
from popoll_backend.query import Query
from popoll_backend.query.get_poll import GetPoll


class UpdatePoll(Query):
    
    name: str
    color: str
    
    def __init__(self, poll: str, name: str, color: str):
        super().__init__(poll)
        self.name = name
        self.color = color
        
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('UPDATE options SET name=?, color=?', (self.name, self.color))
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return GetPoll(self.poll).run(cursor)


from typing import List, Optional
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from tests import IntegrationTest

class TestGetDate(IntegrationTest):
    
    def create_answer_false(self, user_id, date_id):
        id = self.create_answer(user_id=user_id, date_id=date_id)
        return self.update_answer(id, False)

    def setUp(self):
        
        # DATA
        # presence ->| present | absent  | unknown
        #            |         |         |         
        # |instrument|         |         |
        # v          |         |         |
        # -----------+---------+---------+---------
        #      1     |  user1  |  user2  | user3   
        #            | (user4) | (user9) |         
        #            | (user5) |         |         
        #            | (user8) |         |         
        # -----------+---------+---------+---------
        #      2     |  user4  |  user6  | user7
        #            |  user5  | (user2) |
        #            | (user1) | (user9) |
        # -----------+---------+---------+---------
        #      3     |  usr8   |   usr9  | usr10
        #            | (user5) | (user2) | 
        #
        
        
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [self.instru2_id])
        self.user2_id = self.create_user('user2', self.instru1_id, [self.instru2_id, self.instru3_id])
        self.user3_id = self.create_user('user3', self.instru1_id, [])
        
        self.user4_id = self.create_user('user4', self.instru2_id, [self.instru1_id])
        self.user5_id = self.create_user('user5', self.instru2_id, [self.instru1_id, self.instru3_id])
        self.user6_id = self.create_user('user6', self.instru2_id, [])
        self.user7_id = self.create_user('user7', self.instru2_id, [])
        
        self.user8_id = self.create_user('user8', self.instru3_id, [self.instru1_id])
        self.user9_id = self.create_user('user9', self.instru3_id, [self.instru1_id, self.instru2_id])
        self.user10_id = self.create_user('user10', self.instru3_id, [])
        
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        self.answer2_id = self.create_answer_false(user_id=self.user2_id, date_id=self.date1_id)
        # self.answer3_id is None
        self.answer4_id = self.create_answer(self.user4_id, self.date1_id)
        self.answer5_id = self.create_answer(self.user5_id, self.date1_id)
        self.answer6_id = self.create_answer_false(self.user6_id, self.date1_id)
        # self.answer7_id is None
        self.answer8_id = self.create_answer(self.user8_id, self.date1_id)
        self.answer9_id = self.create_answer_false(self.user9_id, self.date1_id)
        # self.answer9_id is None
        

    def _assertAnswer(self, _json, presence: Optional[bool], user_id: int, instrument_id: int, is_main_instrument: bool):
        for answer in _json['answers']:
            if answer['user']['id'] == user_id and answer['instrument']['id'] == instrument_id and answer['is_main_instrument'] == is_main_instrument:
                if (presence == None and answer['answer'] == None) or answer['answer']['response'] == presence:
                    return True
        return False

    def test_stats_globals(self):            
        _json = self.get_date(self.date1_id)
        assert len(_json['answers']) == 19
        
        self._assertAnswer(_json, True, self.user1_id, self.instru1_id, True)
        self._assertAnswer(_json, True, self.user4_id, self.instru1_id, False)
        self._assertAnswer(_json, True, self.user5_id, self.instru1_id, False)
        self._assertAnswer(_json, True, self.user8_id, self.instru1_id, False)
        
        self._assertAnswer(_json, True, self.user4_id, self.instru2_id, True)
        self._assertAnswer(_json, True, self.user5_id, self.instru2_id, True)
        self._assertAnswer(_json, True, self.user1_id, self.instru2_id, False)
        
        self._assertAnswer(_json, True, self.user8_id, self.instru3_id, True)
        self._assertAnswer(_json, True, self.user5_id, self.instru3_id, False)
        
        self._assertAnswer(_json, False, self.user2_id, self.instru1_id, True)
        self._assertAnswer(_json, False, self.user9_id, self.instru1_id, False)
        
        self._assertAnswer(_json, False, self.user6_id, self.instru2_id, True)
        self._assertAnswer(_json, False, self.user2_id, self.instru2_id, False)
        self._assertAnswer(_json, False, self.user9_id, self.instru2_id, False)
        
        self._assertAnswer(_json, False, self.user9_id, self.instru3_id, True)
        self._assertAnswer(_json, False, self.user2_id, self.instru3_id, False)
        
        self._assertAnswer(_json, None, self.user3_id, self.instru1_id, True)
        
        self._assertAnswer(_json, None, self.user7_id, self.instru2_id, True)
        
        self._assertAnswer(_json, None, self.user10_id, self.instru3_id, True)
        


from tests import IntegrationTest

class TestGetUsersSortByName(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('BBB', self.instru2_id, [])
        self.user2_id = self.create_user('AAA', self.instru1_id, [])
        
    def test_get_user_sorted(self):
        _json = self.get_users()
        assert len(_json['users']) == 2
        assert _json['users'][0]['user']['name'] == 'AAA'
        assert _json['users'][0]['user']['id'] == self.user2_id
        assert _json['users'][1]['user']['name'] == 'BBB'
        assert _json['users'][1]['user']['id'] == self.user1_id



from typing import List
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model import Payload


class InstrumentsPayload(Payload):
    
    def __init__(self, instruments: List[Instrument]):
        self.instruments: List[Instrument] = instruments


from typing import List, Optional
from popoll_backend.model.db.answer import Answer
from popoll_backend.model.db.date import Date
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model import Payload
from popoll_backend.model.payload.user import UserOut

class _UserDateOutput():
    
    def __init__(self, user: UserOut, date: Date, answers: List[Answer]):
        self.date: Date = date
        _ans = [a for a in answers if a.user_id == user.user.id and a.date_id == date.id]
        self.answer: Optional[Answer] = _ans[0] if len(_ans) > 0 else None
        

class UserPayload(Payload):
    
    def __init__(self, user: UserOut, answers: List[Answer], dates: List[Date]):
        self.user: UserOut = user
        self.dates: List[_UserDateOutput] = [_UserDateOutput(user, date, answers) for date in dates]


import sqlite3
from typing import List

from popoll_backend.model import Payload
from popoll_backend.model.db.answer import Answer
from popoll_backend.query import Query


class GetAnswer(Query):
    
    id: int
    
    def __init__(self, poll: str, id: int):
        super().__init__(poll)
        self.id = id

    def process(self, cursor: sqlite3.Cursor):
        pass
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return self.selectItem(cursor, 'SELECT * from answers where id=?', self.id, Answer)


from tests import IntegrationTest

class TestGetInstruments(IntegrationTest):

    def setUp(self):
        super().setUp()

    def test_get_instruments(self):            
        _json = self.get_instruments()
        assert len(_json['instruments']) == 3
        assert self.INSTRU1 in [i['name'] for i in _json['instruments']]
        assert self.INSTRU2 in [i['name'] for i in _json['instruments']]
        assert self.INSTRU3 in [i['name'] for i in _json['instruments']]




from tests import IntegrationTest

class TestDeleteUserDateEntries(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.user2_id = self.create_user('user2', self.instru2_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.date2_id = self.create_date(date='2025-03-11', time='18:00:00', title='secondDate')
        self.date3_id = self.create_date(date='2025-03-12', time=None, title='thirdDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        self.answer2_id = self.create_answer(user_id=self.user1_id, date_id=self.date2_id)
        self.answer3_id = self.create_answer(user_id=self.user1_id, date_id=self.date3_id)
        self.answer4_id = self.create_answer(user_id=self.user2_id, date_id=self.date1_id)
        self.answer5_id = self.create_answer(user_id=self.user2_id, date_id=self.date2_id)
        self.answer6_id = self.create_answer(user_id=self.user2_id, date_id=self.date3_id)
        
    def test_delete_user_date_entries(self):
        _json = self.get_users()
        assert len(_json['users']) == 2
        _json_user1 = self.get_user(self.user1_id)
        assert len(_json_user1['dates']) == 3
        assert _json_user1['dates'][0].get('answer', None) != None
        
        self.delete_date(self.date1_id)
        _json = self.get_user(self.user1_id)
        assert len(_json['dates']) == 2
        
        self.delete_user(self.user1_id)
        _json = self.get_users()
        assert len(_json['users']) == 1
        assert _json['users'][0]['user']['id'] == self.user2_id
        
        user3_id = self.create_user('user3', self.instru3_id, [])
        assert self.user1_id != user3_id
        _json = self.get_users()
        assert len(_json['users']) == 2
        _json = self.get_user(user3_id)
        assert _json['dates'][1].get('answer', None) == None
        
        answer7_id = self.create_answer(user_id=user3_id, date_id=self.date2_id)
        self.update_answer(answer7_id, False)
        _json = self.get_user(user3_id)
        assert len(_json['dates']) == 2
        assert _json['dates'][0]['answer']['response'] == False



from typing import Any, List

from popoll_backend.model import Payload


class User(Payload):
    
    def __init__(self, data: List[Any]):
        self.id: int = data[0]
        self.name: str = data[1]

    @classmethod
    def create_table(cls):
        return """ CREATE TABLE IF NOT EXISTS users (
            id integer PRIMARY KEY AUTOINCREMENT,
            name text NOT NULL UNIQUE
        ); """


from tests import IntegrationTest, TestForwardError

class TestUpdateuserEntry(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.user3_id = self.create_user('user3', self.instru1_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        
    def test_update_user_entry(self):
        updated_user1_id = self.update_user(self.user1_id, 'user2', self.instru2_id, [])
        assert updated_user1_id == self.user1_id
        _json = self.get_user(updated_user1_id)
        assert _json['user']['user']['name'] == 'user2'
        assert _json['user']['main_instrument']['id'] == self.instru2_id
        assert _json['user']['main_instrument']['name'] == self.INSTRU2

    def test_update_user_entry_conflict(self):
        rs = self.update_user(self.user1_id, 'user3', self.instru2_id, [], fail=True)
        assert rs.status_code == 400
        # TODO assert json.loads(rs.json)["message"] == 'User with name user1 already exists.'
      
    # TODO Does not make sens since is returning valid answer but nothing impacted  
    # def test_update_user_entry_not_found(self):
    #     try:
    #         rs = self.update_user(self.user1_id + 10, 'user3', self.instru2_id, [], fail=True)
    #         print(rs)
    #         assert False
    #     except TestForwardError as e:
    #         assert e.rs.status_code == 404
    #         _json = self.toJSON(self.get())
    #         assert len(_json['users']) == 2


from typing import List, Optional
from popoll_backend.model.db.answer import Answer
from popoll_backend.model.db.date import Date
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model import Payload

def getUser(users: List[User], id: int) -> User:
    return [user for user in users if user.id == id][0]

def getInstrument(instruments: List[Instrument], id: int) -> Instrument:
    return [instrument for instrument in instruments if instrument.id == id][0]

def getMainInstrument(instruments: List[Instrument], user_instruments: List[UserInstruments], user: User) -> Instrument:
    return getInstrument(instruments, [user_instrument.instrument_id for user_instrument in user_instruments if user_instrument.user_id == user.id and user_instrument.is_main][0])

def getSecondInstruments(instruments: List[Instrument], user_instruments: List[UserInstruments], user: User) -> List[Instrument]:
    return [getInstrument(instruments, iid) for iid in [user_instrument.instrument_id for user_instrument in user_instruments if user_instrument.user_id == user.id and not user_instrument.is_main]]

def getAnswer(answers: List[Answer], user_id: int):
    res: List[Answer] = [answer for answer in answers if answer.user_id == user_id]
    if len(res) == 0:
        return None
    if len(res) > 1:
        print('WEIRD CASE')
    return res[0]

class _DateAnswerPayload:
    
    def __init__(self, answer: Answer, user: User, instrument: Instrument, is_main_instrument: bool):
        self.answer = answer
        self.user = user
        self.instrument = instrument
        self.is_main_instrument = is_main_instrument

class DatePayload(Payload):
    
    def __init__(self, date: Date, answers: List[Answer], users: List[User], instruments: List[Instrument], user_instruments: List[UserInstruments]):
        self.date: Date = date
        self.answers: List[_DateAnswerPayload] = []
        for user in users:
            answer = getAnswer(answers, user.id)
            self.answers.append(_DateAnswerPayload(answer, user, getMainInstrument(instruments, user_instruments, user), True))
            for instru in getSecondInstruments(instruments, user_instruments, user):
                self.answers.append(_DateAnswerPayload(answer, user, instru, False))
        


from tests import IntegrationTest

class TestDBCreation(IntegrationTest):

    def test_db_creation(self):
        _json = self.get_instruments()
        
        assert _json['instruments'][0]['id'] == self.instru1_id
        assert _json['instruments'][0]['name'] == self.INSTRU1
        assert _json['instruments'][1]['id'] == self.instru2_id
        assert _json['instruments'][1]['name'] == self.INSTRU2
        assert _json['instruments'][2]['id'] == self.instru3_id
        assert _json['instruments'][2]['name'] == self.INSTRU3
